# Extending Fine and Gray's model: Weighted NPMLE for the subdistribution hazard and the marginal mean intensity 
# submission # JASA-TM-2014-0543
##################################################################################
# R code for weighted NPMLE, simulation study for the proportional odds model 
################################################################################
rm(list=ls())
set.seed(666)
library(splines)
library(survival)
library(cmprsk)
library(parallel)
library(numDeriv)
n=200
p=0.3
numcov=2
tau=1.4
beta.true=matrix(c(0.5,0.5,-0.5,0.5),nrow=2)  
beta1<-beta.true[1,]
beta2<-beta.true[2,]
#
z1<-rnorm(n)
z2<-rnorm(n)
U <- runif(n)
C<-runif(n,min=0.75,max=1.5)
#################################################################################
cova1<-cbind(z1,z2)
colnames(cova1)<-c("z1","z2")
numcov<-length(cova1[1,])
#################################################################################
# simulate event times from the conditional distibution P(T_i<=t|eps_i=j,Z_i)
#################################################################################
eps <-rep(0,1)
a1 <- cova1%*%beta1
a2 <- cova1%*%beta2
pro <- exp(p+a1)/(1+exp(p+a1))
for (i in (1:n)){eps[i] <- sample(x=c(1,2),size=1,replace=TRUE, prob=c(pro[i],1-pro[i]))}
#####################################################################################
# calculate event times for event type 1 and 2
#####################################################################################
etime = ifelse(eps==1, -log(1-U/(1+(1-U)*exp(p+a1))), -(1/exp(a2))*log(1-U)) 
time<-ifelse(C<etime,C,etime)
status=ifelse(etime<C,eps,0)
status0=ifelse (C<etime,1,0)
status1=ifelse(status==1,1,0)
status2=ifelse(status==2,1,0)
numcens=sum(status0)
numc=numcens
num1=sum(status1)
num2=sum(status2)
##################################################################################
# define simple data frame
##################################################################################
mydata<-cbind(status,status0,status1,status2,time,z1,z2)
colnames(mydata)<-c("status","status0","status1","status2","time","z1","z2")
mydata<-data.frame(mydata)
##################################################################################
# create data frame with ordered event times
##################################################################################
order.time <- order(mydata$time)
Ma <- mydata[order.time,]
rownames(Ma) <- 1:nrow(Ma)
##################################################################################
# KM-estimator of censoring distribution, data frame and IPCW weights
##################################################################################
kmc=rep(1,n)
kmc[1]=(1-(1/n))^{(Ma$status[1]==0)}
for (i in 2:n){kmc[i]<-kmc[i-1]*(1-(1/(n-i+1)))^{(Ma$status[i]==0)}}    
###############################################################################
ind<-c(1:n)
####################################################################################
M<-cbind(Ma$status,Ma$status0,Ma$status1,Ma$status2,kmc,Ma$time,Ma$z1,Ma$z2,ind)
colnames(M)<-c("status","status0","status1","status2","kmc","time","z1","z2","ind")
M<-data.frame(M)
numbig<-ifelse(M$kmc<.05,1,0)
nbig<-sum(numbig)
#
if(nbig>0){
v.salt<-as.vector(seq(tau+0.00001,(tau+nbig*0.00001),by=0.00001))
M$time[(n-nbig+1):n]<-v.salt
M$status[(n-nbig+1):n]<-0
M$status0[(n-nbig+1):n]=1
M$status1[(n-nbig+1):n]=0
M$status2[(n-nbig+1):n]=0
M$kmc[(n-nbig+1):n]=M$kmc[n-nbig]
}
#
numbig2<-ifelse(M$time>tau,1,0)
nbig2<-sum(numbig2)
#
if(nbig2>0){
v.salt2<-as.vector(seq(tau+0.00001,(tau+nbig2*0.00001),by=0.00001))
M$time[(n-nbig2+1):n]<-v.salt2
M$status[(n-nbig2+1):n]<-0
M$status0[(n-nbig2+1):n]=1
M$status1[(n-nbig2+1):n]=0
M$status2[(n-nbig2+1):n]=0
M$kmc[(n-nbig2+1):n]=M$kmc[n-nbig2]
}
M<-data.frame(M)
# 
num1=sum(M$status1)
num2=sum(M$status2)
numcens=sum(M$status0)
numc<-numcens
percens=numc/n
perc2<-num2/n
######################################################################################
# define datasets for type 1 events, type 2 events, censored, censored+type 2
###############################################################################
M1<-subset(M,status==1)
M02<-subset(M,status!=1)
n02<-length(M02[,1])
M2<-subset(M02,status==2)
Mc<-subset(M02,status==0)
######################################################################################
# calculate the matrix of covariates 
######################################################################################
cova<-cbind(M$z1,M$z2)
cov1<-cbind(M1$z1,M1$z2)
cov2<-cbind(M2$z1,M2$z2)
cov02<-cbind(M02$z1,M02$z2)
covc<-cbind(Mc$z1,Mc$z2)
num1=length(M1[,1])
num02=length(M02[,1])
######################################################################################
# caclulate weight
w.new<-matrix(rep(0,num2*num1),nrow=num2)
for (i in 1:num2){w.new[i,]<-(M1$ind>M2$ind[i])*(M1$kmc/M2$kmc[i])}
#############################################################################################
#############################################################################################
Ma3 <- matrix(rep(0,num1*num1),nrow=num1)
for(i in 1:num1){for (k in 1:i){Ma3[i,k]=1}}
M4<- matrix(rep(0,n*num1),nrow=n)
for (k in 1:num1){for(i in M1$ind[k]:n){M4[i,k]<-1}}
M5<-M4
for (i in n:1){if (M$status0[i]==0){M5<-M5[-i,]}}
M6<-M4
for (i in n:1){if (M$status2[i]==0){M6<-M6[-i,]}}
#############################################################################################
fn <- function(x){
  beta <- x[1:numcov]
  alpha <- x[(numcov+1):(numcov+num1)]
  lambda <- exp(alpha)
  #
  Lambda<-rowSums(Ma3%*%lambda)
  Lam.all<-rowSums(M4%*%lambda)
  Lam=t(matrix(rep(Lambda,num2),nrow=num1))
  Lamb<-t(matrix(rep(lambda,num2),nrow=num1))
  V5 <- matrix(rep(exp(cov2%*%beta),num1),nrow=num2)
  #
  f1 <- sum(log(lambda))
  f2 <- sum(cov1%*%beta)
  f3 <- -sum(log(1+exp(cov1%*%beta)*Lambda))
  f4 <- sum(log(1+exp(cova%*%beta)*Lam.all))
  f5 <- sum(w.new*V5*Lamb*(1/(1+(V5*Lam))))
  #
  f<- -f1-f2-f3+f4+f5
  return(f)
}
######################################################################
# caculate approximation for jump sizes
#######################################################################
compr=rep(0,num1)
compr[1]=sum(M$status[0:(M1$ind[1])])
for (j in 2:num1){compr[j]=sum(M$status2[(M1$ind[j-1]):(M1$ind[j])])}
for (j in 1:num1){compr[j]=compr[j]/M$kmc[M1$ind[j]]}  
typ2=(Ma3%*%compr)*M1$kmc  
#######################################################################
sbeta=c(0,0)
salpha=log(1/((n-M1$ind)+typ2))
par<-c(sbeta,salpha)
#
result <- optim(par,fn=fn,gr=NULL,method="BFGS")
estpar<-c(result$par[1:numcov],exp(result$par[(numcov+1):(numcov+num1)]))
#
t1<-sum(ifelse(M1$time<=tau,1,0))
t4<-sum(ifelse(M1$time<=tau/4,1,0))
t2<-sum(ifelse(M1$time<=tau/2,1,0))
# 
latau1<-sum(estpar[(numcov+1):(numcov+t1)])  
latau2<-sum(estpar[(numcov+1):(numcov+t2)]) 
latau4<-sum(estpar[(numcov+1):(numcov+t4)])
#######################################################################
para <-round(c(estpar[1:numcov],latau4,latau2,latau1),5)
lambda<-estpar[(numcov+1):(numcov+num1)]
beta<-estpar[1:numcov]
Lambda<-Ma3%*%lambda
#################################################################################
# Calculate Hessian matrix with respect to jump sizes and beta (backtransform estimates!!!)
###############################################################################################
fback <- function(x){
  beta <- x[1:numcov]
  lambda <- x[(numcov+1):(numcov+num1)]
  #
  Lambda<-Ma3%*%lambda
  Lam.all<-rowSums(M4%*%lambda)
  MLam=t(matrix(rep(Lambda,num2),nrow=num1))
  Mlamb<-t(matrix(rep(lambda,num2),nrow=num1))
  V5=matrix(rep(exp(cov2%*%beta),num1),nrow=num2)
  #
  f1 <- sum(log(lambda))
  f2 <- sum(cov1%*%beta)
  f3 <- -sum(log(1+exp(cov1%*%beta)*Lambda))
  f4 <- sum(log(1+exp(cova%*%beta)*Lam.all))
  f5 <- sum(w.new*V5*Mlamb*(1/(1+(V5*MLam))))
  #
  fback<- -f1-f2-f3+f4+f5
  return(fback)
}
#############################################################################
Mhe <- hessian(func=fback, x=estpar)
breadi<-solve(Mhe)
# define Matrix that transforms vector of jump sizes to vector of cumulative jump sizes
Mtrafo<-rbind(cbind(diag(numcov),matrix(rep(0,numcov*num1),nrow=numcov)),cbind(matrix(rep(0,num1*numcov),nrow=num1),Ma3))
###########################################################################################
#############################################################################
breadinv<-Mtrafo%*%breadi%*%t(Mtrafo)
breadr<-round(sqrt(diag(breadinv)),5)
var.bread<-c(breadr[1:numcov],breadr[t4+numcov],breadr[t2+numcov],breadr[t1+numcov])
######################################################################################
###########################################################################################
# sandwich estimator
###########################################################################################
# gradient for each observation, square and aggregate...
##########################################################################
grad<-array(0,dim=c(numcov+num1,numcov+num1,n))
gradi<-array(0,c(n,numcov+num1))
#########################################################################
lambda<-estpar[(numcov+1):(numcov+num1)]
beta<-estpar[1:numcov]
Lambda<-Ma3%*%lambda
Lamc<-rowSums(M5%*%lambda)
Lam2<-rowSums(M6%*%lambda)
vecii<-c(1:num1)
# define G'(exp(cov1*beta^T*A(.))))
MGgrad1<-1/(1+exp(cov1%*%beta)*Lambda)
# define G'(exp(covc*beta^T*A(.))))
MGgradc<-1/(1+exp(covc%*%beta)*Lamc)
# individuals with events of type 1
for(k in 1:numcov){gradi[M1$ind,k]<-(-1)*(cov1[,k]*(1-2*(exp(cov1%*%beta)*Lambda)*MGgrad1))}
for(k in 1:num1){gradi[M1$ind,numcov+k]<-(-1)*((vecii==k)/lambda[k]-2*(k<=vecii)*(exp(cov1%*%beta))*MGgrad1)}
# censored individuals
for(k in 1:numcov){gradi[Mc$ind,k]<-(covc[,k]*exp(covc%*%beta)*Lamc)*MGgradc}
for(k in 1:num1){gradi[Mc$ind,numcov+k]<-(M1$ind[k]<=Mc$ind)*exp(covc%*%beta)*MGgradc}
#
# individuals with events of type 2
Mglami<-t(matrix(rep(lambda,num2),nrow=num1))
MGLami<-t(matrix(rep(Lambda,num2),nrow=num1))
# define G'(exp(cov2[i,]*beta^T*A(X_i)))
MGgrad2<-1/(1+exp(cov2%*%beta)*Lam2)
# define G''(exp(cov2[i,]*beta^T*A(X_i)))
MGgrad2.2<-(-1)/(1+exp(cov2%*%beta)*Lam2)^2
vgrbet<-exp(cov2%*%beta)
Mgcov2b<-matrix(rep(vgrbet,num1),nrow=num2)
Mgradin<-Mgcov2b*MGLami
# define matrix of G'(e^{beta^TZ_i}A(X_k))
MG1grad2<-1/(1+Mgradin)
# define matrix of G''(e^{beta^TZ_i}A(X_k))
MG2grad2<-(-1)/(1+Mgradin)^2
#
MGgra1<-w.new*Mglami*(MG1grad2+MGLami*Mgcov2b*MG2grad2)
#
Mgrb1<-w.new*MG1grad2
Mgrb2<-w.new*MG2grad2*Mglami
#  
for(l in 1:numcov){gradi[M2$ind,l]<-(cov2[,l]*exp(cov2%*%beta)*Lam2)*MGgrad2+cov2[,l]*vgrbet*rowSums(MGgra1)}
for(l in 1:(num1-1)){gradi[M2$ind,(numcov+l)]<-(M1$ind[l]<=M2$ind)*exp(cov2%*%beta)*MGgrad2+vgrbet*Mgrb1[,l]+(vgrbet^2)*rowSums(Mgrb2[,l:num1])}
gradi[M2$ind,(numcov+num1)]<-(M1$ind[num1]<=M2$ind)*exp(cov2%*%beta)*MGgrad2+vgrbet*Mgrb1[,num1]+(vgrbet^2)*Mgrb2[,num1]
##################################################################################
for (i in 1:n){grad[,,i]<-(gradi[i,])%*%t(gradi[i,])}
##################################################################################
# comupute sandwich estimator
#########################################################################
meat<-apply(grad,1:2,sum)
sandw <- (Mtrafo)%*%(breadi)%*%(meat)%*%t(breadi)%*%t(Mtrafo)
sandwich<-round(sqrt(diag(sandw)),5)
var.sand<-c(sandwich[1:numcov],sandwich[t4+numcov],sandwich[t2+numcov],sandwich[t1+numcov])
###########################################################################################################
# sandwich including correction term
#####################################################################################################
# caclulate martingale dM^c_i
Mus1<-matrix(rep(0,n*n),nrow=n)
diag(Mus1)<-M$status0
head(M)
#
hazn<-(M$status==0)/(n-M$ind+1)
Mus2<-matrix(rep(0,n*n),nrow=n)
for (i in 1:n){Mus2[i,]<-hazn*(M$ind[i]>=M$ind)}
#
Mu<-Mus1-Mus2
####################################################################################################
# estimates for the hazard, beta and cumulative hazard
#############################################################################################
MLam<-t(matrix(rep(Lambda,num2),nrow=num1))
Mlam<-t(matrix(rep(lambda,num2),nrow=num1))
Mlamn<-t(matrix(rep(lambda,n),nrow=n))
Mzbet<-matrix(rep(exp(cov2%*%beta),num1),nrow=num2)
Mzbetn<-t(matrix(rep(exp(cov2%*%beta),n),nrow=n))
# Define Matrix e^{beta^TZ_j}A(X_k)=Mzbet[]
MzbL<-Mzbet*MLam
# Define matrix G'(e^{beta^TZ_j}A(X_k))- proportional odds model
MG1<- 1/(1+MzbL)
# Define matrix G''(e^{beta^TZ_j}A(X_k))
MG2<- -1/(1+MzbL)^2
#######################################################################
# Define first part of residual term, for beta
Mjk<-w.new*(MG1+Mzbet*MLam*MG2)*Mlam
Muk.ind<-array(0,dim=c(n,num1))
for(u in 1:n){Muk.ind[u,]<-(u<=M1$ind)}
Mju<-array(0,dim=c(num2,n))
for(j in 1:num2){for(u in 1:n){Mju[j,u]<-sum(Mjk[j,]*Muk.ind[u,])}}
Mju.ind<-array(0,dim=c(num2,n))
for(u in 1:n){Mju.ind[,u]<-(M2$ind<=u)}
Mju.new<-Mju*Mju.ind*Mzbetn
#
q1<-array(0,dim=c(numcov,n))
for(l in 1:numcov){for(u in 1:n){q1[l,u]<-sum(cov2[,l]*Mju.new[,u])}}
############################################################################
#second part of residuals for baseline
M2jl<-w.new*Mzbet*MG1
M2jl.ind<-Mju.ind
M2lu.ind<-t(Muk.ind)
M2lu<-array(0,dim=c(num1,n))
for(l in 1:num1){for(u in 1:n){M2lu[l,u]<-sum(M2jl[,l]*M2jl.ind[,u])}}
#
q21<-M2lu.ind*M2lu
############################################################################
# third part of residuals for baseline
M3jk<-Mzbet^2*w.new*MG2
M3ju.ind<-array(0,dim=c(num2,n))
for(u in 1:n){M3ju.ind[,u]<-(M2$ind<=u)}
M3ku<-array(0,dim=c(num1,n))
for(k in 1:num1){for(u in 1:n){M3ku[k,u]<-(u<=M1$ind[k])*sum(M3jk[,k]*M3ju.ind[,u])}}
M3ku.new<-M3ku*Mlamn
M3kl.ind<-array(0,dim=c(num1,num1))
for(l in 1:num1){for (k in l:num1){M3kl.ind[k,l]<-1}}
#
q22<-array(0,dim=c(num1,n))
for(l in 1:num1){for(u in 1:n){q22[l,u]<-sum(M3ku.new[,u]*M3kl.ind[,l])}}
#
q<-rbind(q1,q21+q22)
#####################################################################################################
# define pi(u)
pi<-n-M$ind+1
######################################################################################################
psi<-array(0,dim=c(n,num1+numcov))
for(k in 1:(num1+numcov)){for(i in 1:n){psi[i,k]<-sum((q[k,]/pi)*Mu[i,])}}
###########################################################################################################
grad3<-array(0,dim=c(numcov+num1,numcov+num1,n))
for (i in 1:n){grad3[,,i]<-(gradi[i,]+psi[i,])%*%t(gradi[i,]+psi[i,])}
######################################################################################################
# adjusted sandwich
meat3 <- apply(grad3,1:2,sum)
sandw3<-(Mtrafo)%*%(breadi)%*%(meat3)%*%(breadi)%*%t(Mtrafo)
sandwich3<-round(sqrt(diag(sandw3)),5)
var.sand3<-c(sandwich3[1:numcov],sandwich3[t4+numcov],sandwich3[t2+numcov],sandwich3[t1+numcov])
#########################################################################################################
# Print vector of parameter estimates, sandwich with correction term, inverse Fisher information, simple sandwich estimator
print(round(cbind(para,var.sand3,var.bread,var.sand),3))
# print number of censorings and competing events in percent
print(cbind(percens,perc2))
############################################################################################
